/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "kernel.h"

#ifndef __GNUC__
#pragma -s1
#endif

#include "main.h"
#include "canvas.h"
#include "icons.h"
#include "menu.h"
#include "msg.h"

#define linesize (52)

#define line1	(8)
#define line2	(line1+linesize)
#define line3	(line2+linesize)
#define line4	(line3+linesize)
#define line5	(line4+linesize)
#define top	(line4+linesize+4)
#define top2	(line5+linesize+4)

#define split   (160)

static int canvas_iconflag = 0x0700013d;
static char canvas_validstr[4] = "";

void infobox_3d(void)
{
  canvas_validstr[0] = 'r';
  canvas_validstr[1] = '2';
  canvas_iconflag = 0x1700013d;
}

canvas create_infobox(char * name, char * purpose, char * author, char * version)
{
  canvas c;
  icon_data d;
  int l, m;

  if(get_3dstatus()) infobox_3d();

  l = main_strwidth(purpose);
  m = main_strwidth(name);
  if(m > l) l = m;
  m = main_strwidth(author);
  if(m > l) l = m;
  m = main_strwidth(version);
  if(m > l) l = m;

  l += 32;

  c = canvas_create(split + l, top, "About this program");
  if(c == NULL)
    return c;
  canvas_setflags(c, CANVAS_BACK | CANVAS_CLOSE, 0);
  canvas_make(c);
  strcpy(d.text, "Version:");
  icon_createforgetful(c, 8, line1, split, line2 - 4, 0x17000211, & d);
  strcpy(d.text, "Author:");
  icon_createforgetful(c, 8, line2, split, line3 - 4, 0x17000211, & d);
  strcpy(d.text, "Purpose:");
  icon_createforgetful(c, 8, line3, split, line4 - 4, 0x17000211, & d);
  strcpy(d.text, "Name:");
  icon_createforgetful(c, 8, line4, split, top - 8, 0x17000211, & d);
  d.indirecttext.text = version;
  d.indirecttext.valid = canvas_validstr;
  d.indirecttext.length = 32;
  icon_createforgetful(c, split + 4, line1, l - 8 + split, line2 - 4, canvas_iconflag, & d);
  d.indirecttext.text = author;
  icon_createforgetful(c, split + 4, line2, l - 8 + split, line3 - 4, canvas_iconflag, & d);
  d.indirecttext.text = purpose;
  icon_createforgetful(c, split + 4, line3, l - 8 + split, line4 - 4, canvas_iconflag, & d);
  d.indirecttext.text = name;
  icon_createforgetful(c, split + 4, line4, l - 8 + split, top - 8, canvas_iconflag, & d);

  return c;
}

canvas create_licenseinfobox(char * name, char * purpose, char * author, char * version, char * license)
{
  canvas c;
  icon_data d;
  int l, m;

  if(get_3dstatus()) infobox_3d();

  l = main_strwidth(purpose);
  m = main_strwidth(name);
  if(m > l) l = m;
  m = main_strwidth(author);
  if(m > l) l = m;
  m = main_strwidth(version);
  if(m > l) l = m;
  m = main_strwidth(license);
  if(m > l) l = m;

  l += 32;

  c = canvas_create(split + l, top2, "About this program");
  if(c == NULL)
    return c;
  canvas_setflags(c, CANVAS_BACK | CANVAS_CLOSE, 0);
  canvas_make(c);
  strcpy(d.text, "Version:");
  icon_createforgetful(c, 8, line1, split, line2 - 4, 0x17000211, & d);
  strcpy(d.text, "License:");
  icon_createforgetful(c, 8, line2, split, line3 - 4, 0x17000211, & d);
  strcpy(d.text, "Author:");
  icon_createforgetful(c, 8, line3, split, line4 - 4, 0x17000211, & d);
  strcpy(d.text, "Purpose:");
  icon_createforgetful(c, 8, line4, split, line5 - 4, 0x17000211, & d);
  strcpy(d.text, "Name:");
  icon_createforgetful(c, 8, line5, split, top2 - 8, 0x17000211, & d);
  d.indirecttext.text = version;
  d.indirecttext.valid = canvas_validstr;
  d.indirecttext.length = 32;
  icon_createforgetful(c, split + 4, line1, l - 8 + split, line2 - 4, canvas_iconflag, & d);
  d.indirecttext.text = license;
  icon_createforgetful(c, split + 4, line2, l - 8 + split, line3 - 4, canvas_iconflag, & d);
  d.indirecttext.text = author;
  icon_createforgetful(c, split + 4, line3, l - 8 + split, line4 - 4, canvas_iconflag, & d);
  d.indirecttext.text = purpose;
  icon_createforgetful(c, split + 4, line4, l - 8 + split, line5 - 4, canvas_iconflag, & d);
  d.indirecttext.text = name;
  icon_createforgetful(c, split + 4, line5, l - 8 + split, top2 - 8, canvas_iconflag, & d);

  return c;
}

canvas create_compileinfobox(char * name, char * purpose, char * author, char * version, char * build)
{
  canvas c;
  icon_data d;
  int l, m;

  if(get_3dstatus()) infobox_3d();

  l = main_strwidth(purpose);
  m = main_strwidth(name);
  if(m > l) l = m;
  m = main_strwidth(author);
  if(m > l) l = m;
  m = main_strwidth(version);
  if(m > l) l = m;
  m = main_strwidth(build);
  if(m > l) l = m;

  l += 32;

  c = canvas_create(split + l, top2, "About this program");
  if(c == NULL)
    return c;
  canvas_setflags(c, CANVAS_BACK | CANVAS_CLOSE, 0);
  canvas_make(c);
  strcpy(d.text, "Build:");
  icon_createforgetful(c, 8, line1, split, line2 - 4, 0x17000211, & d);
  strcpy(d.text, "Version:");
  icon_createforgetful(c, 8, line2, split, line3 - 4, 0x17000211, & d);
  strcpy(d.text, "Author:");
  icon_createforgetful(c, 8, line3, split, line4 - 4, 0x17000211, & d);
  strcpy(d.text, "Purpose:");
  icon_createforgetful(c, 8, line4, split, line5 - 4, 0x17000211, & d);
  strcpy(d.text, "Name:");
  icon_createforgetful(c, 8, line5, split, top2 - 8, 0x17000211, & d);
  d.indirecttext.valid = canvas_validstr;
  d.indirecttext.length = 32;
  d.indirecttext.text = build;
  icon_createforgetful(c, split + 4, line1, l - 8 + split, line2 - 4, canvas_iconflag, & d);
  d.indirecttext.text = version;
  icon_createforgetful(c, split + 4, line2, l - 8 + split, line3 - 4, canvas_iconflag, & d);
  d.indirecttext.text = author;
  icon_createforgetful(c, split + 4, line3, l - 8 + split, line4 - 4, canvas_iconflag, & d);
  d.indirecttext.text = purpose;
  icon_createforgetful(c, split + 4, line4, l - 8 + split, line5 - 4, canvas_iconflag, & d);
  d.indirecttext.text = name;
  icon_createforgetful(c, split + 4, line5, l - 8 + split, top2 - 8, canvas_iconflag, & d);

  return c;
}

void return_help(char * txt)
{
  char * a = txt, * b = poll_block.data.chars;
  while(* a > 31) * b ++ = * a ++ ;
  * b = 0;
  poll_block.size = (24 + strlen(poll_block.data.chars)) & 0xfffffffc;
  poll_block.action = 0x503;
  poll_block.your_ref = poll_block.my_ref;
  /* poll_block.my_ref = poll_block.your_ref; */
  main_send(& poll_block, 17, poll_block.sender, 0);
}

void werr(int fatal, char * string, ...)
{
  _kernel_oserror o;
  char title[48];
  va_list valist;

  va_start(valist, string);
  o.errnum = 0;
  vsprintf(o.errmess, string, valist);
  va_end(valist);
  if(fatal) {
    sprintf(title, "Fatal error from %s", TASKNAME);
    new_errorbox(& o, title, ERRORSPRITE, 1, QUIT);
    exit(0);
  } else {
    sprintf(title, "Error from %s", TASKNAME);
    new_errorbox(& o, title, ERRORSPRITE, 1, CONTINUE);
  }
}

void swerr(int fatal, char * string, ...)
{
  _kernel_oserror o;
  char title[48];
  va_list valist;
  _kernel_swi_regs regs;

  va_start(valist, string);
  o.errnum = 0;
  vsprintf(o.errmess, string, valist);
  va_end(valist);
  regs.r[0] = (int) & o;
  regs.r[1] = 17;
  regs.r[2] = (int)title;
  if(fatal) {
    sprintf(title, "Fatal error from %s", TASKNAME);
    _kernel_swi(0x600df, & regs, & regs);
    exit(0);
  } else {
    sprintf(title, "Error from %s", TASKNAME);
    _kernel_swi(0x600df, & regs, & regs);
  }
}

int message_box(int flags, char * title, char * string, ...)
{
  _kernel_oserror o;
  va_list valist;
  _kernel_swi_regs regs;

  va_start(valist, string);
  o.errnum = 0;
  vsprintf(o.errmess, string, valist);
  va_end(valist);
  regs.r[0] = (int) & o;
  regs.r[1] = 16 | flags;
  regs.r[2] = (int)title;
  _kernel_swi(0x600df, & regs, & regs);
  return regs.r[1];
}

static void __null_proc(void)
{
}

void poll_init(void)
{
  do {
    main_addhandler(HANDLER_RESERVED1, __null_proc);
    poll_once();
  } while(poll_lastevent != 0);
}

menu msg_menu(char * msg, int entries, _rov_menuh p1, _rov_submenuh p2)
{
  int i, l;
  menu tmp;
  char str[32];

  strcpy(str, msg);
  strcat(str, "00");
  tmp = menu_create(msg_lookup(str), entries, p1, p2);
  l = strlen(msg);
  for(i = 1; i <= entries; i ++) {
    str[l + 1] ++ ;
    if(str[l + 1] > '9') {
      str[l] ++ ;
      str[l + 1] = '0';
    }
    menu_add(tmp, msg_lookuprom(str));
  }
  return tmp;
}

menu msg_menuwriteable(char * msg, int entries, char * write, char * valid, int len, _rov_menuh p1, _rov_submenuh p2)
{
  int i, l;
  menu tmp;
  char str[12];

  strcpy(str, msg);
  strcat(str, "00");
  tmp = menu_create(msg_lookup(str), entries, p1, p2);
  l = strlen(msg);
  for(i = 1; i < entries; i ++) {
    str[l + 1] ++ ;
    if(str[l + 1] > '9') {
      str[l] ++ ;
      str[l + 1] = '0';
    }
    menu_add(tmp, msg_lookuprom(str));
  }
  menu_addwriteable(tmp, write, valid, len);
  return tmp;
}
